searchData={"items":[{"type":"module","title":"Mozart.Data.ProcessModel","doc":"```\ndefstruct [:name, :tasks, :initial_task]\n```\n\nThis struct is used to create business process models. Once created, a process model\ncan be loaded into a `Mozart.ProcessEngine.html` and executed.\n\nAlso see Mozart.Task modules.\n\nExample:\n\n```\n%ProcessModel{\n      name: :user_task_process_model,\n      tasks: [\n        %User{\n          name: :user_task,\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n    }\n```","ref":"Mozart.Data.ProcessModel.html"},{"type":"module","title":"Mozart.Data.ProcessState","doc":"```\ndefstruct [\n    :uid,\n    :parent,\n    :model_name,\n    :start_time,\n    :end_time,\n    :execute_duration,\n    open_tasks: %{},\n    completed_tasks: [],\n    data: %{},\n    complete: false\n  ]\nend\n```\n\nThis struct is used to represent the state of a `Mozart.ProcessEngine` execution.\n\nExample: (incrementally populated throughout process execution.)\n\n```\n%Mozart.Data.ProcessState{\n  uid: \"74146e68-088e-42b6-965e-20f4d7dbae16\",\n  parent: nil,\n  model_name: :call_external_service,\n  start_time: ~U[2024-05-27 15:22:01.847683Z],\n  end_time: ~U[2024-05-27 15:22:02.277273Z],\n  execute_duration: 429590,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      data: %{},\n      function: #Function<0.23164178/1 in Mozart.ProcessModels.TestModels.call_exteral_services/0>,\n      name: :get_api_data,\n      type: :service,\n      next: nil,\n      __struct__: Mozart.Task.Service,\n      uid: \"e3326041-d203-46b2-8141-907f71421398\",\n      process_uid: \"74146e68-088e-42b6-965e-20f4d7dbae16\"\n    }\n  ],\n  data: %{\n    todo_data: %{\n      \"completed\" => false,\n      \"id\" => 1,\n      \"title\" => \"delectus aut autem\",\n      \"userId\" => 1\n    }\n  },\n  complete: true\n}\n```","ref":"Mozart.Data.ProcessState.html"},{"type":"module","title":"Mozart.ProcessEngine","doc":"A ProcessEngine is dynamically spawned for the purpose of executing a `Mozart.Data.ProcessModel`.","ref":"Mozart.ProcessEngine.html"},{"type":"function","title":"Mozart.ProcessEngine.execute/1","doc":"Used to complete any \"complete-able\" open tasks. Task execution frequently spawns new\nopen tasks. Execute will continue to as long as there are \"complete-able\" open tasks.\n\nNote: Some types of task are complete-able immediately and some are not. For\nexample:\n  * A `Mozart.Task.Service` task is complete-able as soon as it is opened.\n  * A `Mozart.Task.User` task when a user completes the task.\n  * A `Mozart.Task.Receive` task is complete-able when a matching\n    `Mozart.Task.Send` task is received.","ref":"Mozart.ProcessEngine.html#execute/1"},{"type":"function","title":"Mozart.ProcessEngine.start_process/3","doc":"Use this function to create a ProcessEngine instance initialized with the\nname of the process model to be executed and any initialization data. The\nengine will start executing tasks with the execute/1 function is called.","ref":"Mozart.ProcessEngine.html#start_process/3"},{"type":"module","title":"Mozart.ProcessModelService","doc":"Acts as a reposatory for storing and retreiving `Mozart.Data.ProcessModel`","ref":"Mozart.ProcessModelService.html"},{"type":"function","title":"Mozart.ProcessModelService.get_process_model/1","doc":"Retrieves a process model by name.","ref":"Mozart.ProcessModelService.html#get_process_model/1"},{"type":"function","title":"Mozart.ProcessModelService.load_process_model/1","doc":"Loads a single process model in the repository.","ref":"Mozart.ProcessModelService.html#load_process_model/1"},{"type":"function","title":"Mozart.ProcessModelService.load_process_models/1","doc":"Loads a list of `Mozart.Data.ProcessModel`s into the state of the\nProcessModelService.","ref":"Mozart.ProcessModelService.html#load_process_models/1"},{"type":"module","title":"Mozart.ProcessService","doc":"This modeule provides services required by individual `Mozart.ProcessEngine` instances. Currently,\nit has no user level functions. Subject to change.","ref":"Mozart.ProcessService.html"},{"type":"function","title":"Mozart.ProcessService.get_completed_process/1","doc":"Returns the state of the completed process corresponding to the process engine's uid","ref":"Mozart.ProcessService.html#get_completed_process/1"},{"type":"function","title":"Mozart.ProcessService.get_user_task/1","doc":"Get a user task by uid","ref":"Mozart.ProcessService.html#get_user_task/1"},{"type":"function","title":"Mozart.ProcessService.get_user_tasks_for_groups/1","doc":"Returns the user tasks that can be completed by users belonging to one of the input groups.","ref":"Mozart.ProcessService.html#get_user_tasks_for_groups/1"},{"type":"function","title":"Mozart.ProcessService.get_user_tasks_for_user/1","doc":"Get user tasks eligible for assignment and completion by the specified user.","ref":"Mozart.ProcessService.html#get_user_tasks_for_user/1"},{"type":"function","title":"Mozart.ProcessService.insert_completed_process/1","doc":"","ref":"Mozart.ProcessService.html#insert_completed_process/1"},{"type":"module","title":"Mozart.Task.Choice","doc":"Used to define a process model Choice Task. Serves the same purpose as a BPMN2 Exclusive Gate.\nThe **choices** field is used to specify the multiple execution paths.\n\nExample:\n```\n%ProcessModel{\n      name: :choice_process_model,\n      tasks: [\n        %Choice{\n          name: :choice_task,\n          choices: [\n            %{\n              expression: fn data -> data.value < 10 end,\n              next: :foo\n            },\n            %{\n              expression: fn data -> data.value >= 10 end,\n              next: :bar\n            }\n          ]\n        }\n      ],\n      initial_task: :choice_task\n    }\n```","ref":"Mozart.Task.Choice.html"},{"type":"module","title":"Mozart.Task.Join","doc":"Used to join parallel execution paths.\n\nExample:\n\n```\n%ProcessModel{\n      name: :parallel_process_model,\n      tasks: [\n        %Parallel{\n          name: :parallel_task,\n          multi_next: [:foo, :bar]\n        },\n        %Service{\n          name: :foo,\n          function: fn data -> Map.merge(data, %{foo: :foo}) end,\n          next: :join_task\n        },\n        %Service{\n          name: :bar,\n          function: fn data -> Map.merge(data, %{bar: :bar}) end,\n          next: :foo_bar\n        },\n        %Service{\n          name: :foo_bar,\n          function: fn data -> Map.merge(data, %{foo_bar: :foo_bar}) end,\n          next: :join_task\n        },\n        # Here's the Join task. Must receive all inputs before completing.\n        %Join{\n          name: :join_task,\n          inputs: [:foo, :foo_bar],\n          next: :final_service\n        },\n        %Service{\n          name: :final_service,\n          function: fn data -> Map.merge(data, %{final: :final}) end\n        }\n      ],\n      initial_task: :parallel_task\n    }\n```","ref":"Mozart.Task.Join.html"},{"type":"module","title":"Mozart.Task.Parallel","doc":"Use to create paralled execution paths.\n\nExample:\n\n```\n%ProcessModel{\n      name: :parallel_process_model,\n      tasks: [\n        # Here is the parallel task. Both :foo and :bar start paralled paths\n        %Parallel{\n          name: :parallel_task,\n          multi_next: [:foo, :bar]\n        },\n        %Service{\n          name: :foo,\n          function: fn data -> Map.merge(data, %{foo: :foo}) end,\n          next: :join_task\n        },\n        %Service{\n          name: :bar,\n          function: fn data -> Map.merge(data, %{bar: :bar}) end,\n          next: :foo_bar\n        },\n        %Service{\n          name: :foo_bar,\n          function: fn data -> Map.merge(data, %{foo_bar: :foo_bar}) end,\n          next: :join_task\n        },\n        %Join{\n          name: :join_task,\n          inputs: [:foo, :foo_bar],\n          next: :final_service\n        },\n        %Service{\n          name: :final_service,\n          function: fn data -> Map.merge(data, %{final: :final}) end\n        }\n      ],\n      initial_task: :parallel_task\n    }\n```","ref":"Mozart.Task.Parallel.html"},{"type":"module","title":"Mozart.Task.Receive","doc":"Used to model a task that waits for a PubSub message. Functional but needs\nsome work.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_receive_task,\n      tasks: [\n        %Receive{\n          name: :receive_task,\n          message_selector: fn msg ->\n            case msg do\n              :message -> %{message: true}\n              _ -> false\n            end\n          end\n        }\n      ],\n      initial_task: :receive_task\n    },\n    %ProcessModel{\n      name: :process_with_single_send_task,\n      tasks: [\n        %Send{\n          name: :send_task,\n          message: :message\n        },\n      ],\n      initial_task: :send_task\n    }\n```","ref":"Mozart.Task.Receive.html"},{"type":"module","title":"Mozart.Task.Rule","doc":"Used to model a Mozart run task. Called a Business Rule in BPMN2. Uses\nthe Elixir [rule_table](https://hex.pm/packages/rule_table) library.\n\nExample:\n\n```\n%ProcessModel{\n      name: :load_approval,\n      tasks: [\n        %Decision{\n          name: :loan_decision,\n          input_fields: [:income],\n          rule_table:\n            Tablex.new(\"\"\"\n            F     income      || status\n            1     > 50000     || approved\n            2     <= 49999    || declined\n            \"\"\"),\n        },\n      ],\n      initial_task: :loan_decision\n    }\n```","ref":"Mozart.Task.Rule.html"},{"type":"module","title":"Mozart.Task.Send","doc":"Used to send a PubSub message to a waiting Receive task. Functioal but needs\nsome work.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_receive_task,\n      tasks: [\n        %Receive{\n          name: :receive_task,\n          message_selector: fn msg ->\n            case msg do\n              :message -> %{message: true}\n              _ -> false\n            end\n          end\n        }\n      ],\n      initial_task: :receive_task\n    },\n    %ProcessModel{\n      name: :process_with_single_send_task,\n      tasks: [\n        %Send{\n          name: :send_task,\n          message: :message\n        },\n      ],\n      initial_task: :send_task\n    }\n```","ref":"Mozart.Task.Send.html"},{"type":"module","title":"Mozart.Task.Service","doc":"Used to model a Service task. A service task calls a function and returns\ndata that is into the state data.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_single_service_task,\n      tasks: [\n        %Service{\n          name: :service_task,\n          input_fields: [],\n          function: fn data -> Map.merge(data, %{single_service: true}) end\n        }\n      ],\n      initial_task: :service_task\n    }\n```","ref":"Mozart.Task.Service.html"},{"type":"function","title":"Mozart.Task.Service.__struct__/0","doc":"","ref":"Mozart.Task.Service.html#__struct__/0"},{"type":"module","title":"Mozart.Task.Subprocess","doc":"Used to model a task that is completed by executing and completing a\nsubprocess. Called a **call activity** in BPMN2.\n\nExample:\n\n```\n%ProcessModel{\n      name: :call_process_model,\n      tasks: [\n        %Subprocess{\n          name: :call_process_task,\n          sub_process: :service_subprocess_model,\n          next: :service_task1\n        },\n        %Service{\n          name: :service_task1,\n          function: fn data -> Map.put(data, :value, data.value + 1) end\n        }\n      ],\n      initial_task: :call_process_task\n    },\n    %ProcessModel{\n      name: :service_subprocess_model,\n      tasks: [\n        %Service{\n          name: :service_task,\n          function: fn data -> Map.put(data, :subprocess_data, \"subprocess data\") end\n        }\n      ],\n      initial_task: :service_task\n    }\n```","ref":"Mozart.Task.Subprocess.html"},{"type":"module","title":"Mozart.Task.Timer","doc":"Used to model a task waits a specified duration and automatically completes.\n\nExample:\n\n```\n%ProcessModel{\n    name: :call_timer_task,\n    tasks: [\n      %Timer{\n        name: :wait_1_seconds,\n        timer_duration: 10\n      },\n    ],\n    initial_task: :wait_1_seconds\n  }\n```","ref":"Mozart.Task.Timer.html"},{"type":"module","title":"Mozart.Task.User","doc":"Used to model a task that must be completed by a system user.\n\nExample:\n\n```\n%ProcessModel{\n      name: :user_task_process_model,\n      tasks: [\n        %User{\n          name: :user_task,\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n    }\n```","ref":"Mozart.Task.User.html"},{"type":"module","title":"Mozart.UserService","doc":"Provides client applications with functionality related to users and groups. Much more\nfunctionality to come.","ref":"Mozart.UserService.html"},{"type":"function","title":"Mozart.UserService.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Mozart.UserService.html#child_spec/1"},{"type":"function","title":"Mozart.UserService.get_assigned_groups/1","doc":"Returns the groups that a user is a member of.","ref":"Mozart.UserService.html#get_assigned_groups/1"},{"type":"function","title":"Mozart.UserService.get_user/1","doc":"Given a user id, return detailed user info.","ref":"Mozart.UserService.html#get_user/1"},{"type":"function","title":"Mozart.UserService.insert_user/1","doc":"Adds a new user to the system.","ref":"Mozart.UserService.html#insert_user/1"},{"type":"extras","title":"Introduction to BPM","doc":"# Introduction to BPM\n\nThe next several sections provide an introduction to to ***Business Process Management***.","ref":"intro_bpm.html"},{"type":"extras","title":"What is a Business Process? - Introduction to BPM","doc":"Companies large and small have business processes that they regularly execute. Let's just call them \"processes\" for brevity's sake. Insurance companies have processes to process a claim. Banks have processes to approve loans. The government have processes to award contracts. \n\nBusiness processes have the following charateristics:\n\n* A business process is comprised of some number of individual tasks. \n* Tasks are performed in an order prescribed by network of task connections.\n* Processes can branch to multiple alternative sets of tasks depending on the state of the process.\n* Processes can branch to multiple parallel paths as well.\n* There are multiple kinds of tasks. Some tasks are performed by users. Some tasks are performed by calling services. There are many other kinds of tasks as well.\n* Processes store data to be used by tasks to perform their functions.\n* Processes allow tasks to add data for use by downstream tasks.","ref":"intro_bpm.html#what-is-a-business-process"},{"type":"extras","title":"What is Business Process Management (BPM)? - Introduction to BPM","doc":"Business Process Management (BPM) is the organized and directed activity of a company to control, monitor and improve the quality of their business processes. They do this with the aid of a BPM platform made available by dozens of commercial vendors.\n\nBPM platforms provide the following functions:\n\n* Provide developers with the ability of define and store process models.\n* Provide developers with the ability have certain tasks completed by making service calls.\n* Provide users and administrators with the ability to start process instances.\n* Provide users with the ability to assign user tasks to specified users.\n* Prvide users with the ability to assign user tasks to members of a specified user group.\n\nThe goals of Business Process Management are:\n\n* Better Process Reliablibility\n* Faster Process Execugtion\n* Reduced Training Requirements\n* Incremental Process Improvement\n* Provide Process Metrics\n* Cost Savings","ref":"intro_bpm.html#what-is-business-process-management-bpm"},{"type":"extras","title":"When is BPM Applicable - Introduction to BPM","doc":"This is difficult to answer exactly, but BPM may be applicable when:\n\n* Processes are performed on a regular basis.\n* Successful process execution is crucial to the business.\n* Current processes are error prone.\n* The speed of process completion is very important.\n* Processes are well defined.\n* Multiple business groups and/or people participate in process execution.","ref":"intro_bpm.html#when-is-bpm-applicable"},{"type":"extras","title":"Introduction to Mozart","doc":"# Introduction to Mozart","ref":"app_overview.html"},{"type":"extras","title":"Process Models & Engines - Introduction to Mozart","doc":"One of the primary concepts of *Mozart* is the **Process Model** implemented by `Mozart.Data.ProcessModel`. A *process model* is an executable representation of a business process. \n\nIf you are a developer working for a bank, you might be asked to develop a process model for handling a customer loan application.\n\nBut what does it mean for a process model to be executable? That's where another major concept comes in - the **Process Engine** implemented by `Mozart.ProcessEngine`. A process model and some initial data are loaded into a process engine, after which process execution initiated.","ref":"app_overview.html#process-models-engines"},{"type":"extras","title":"Process Model Tasks - Introduction to Mozart","doc":"What happens when a process engine executes a process model? That brings us to the notion of **Tasks**. Tasks are defined in the process model. Each task represents some work that needs to be completed. The job of the process engine is to drive completion of all of the tasks specified in the process model.\n\nMozart provides arounds a dozen different type of *tasks*. The two most ofen used are the `Mozart.Task.User` task and the `Mozart.Task.Service` task.","ref":"app_overview.html#process-model-tasks"},{"type":"extras","title":"Process Model Service - Introduction to Mozart","doc":"When a process model has been developed, it is stored in a process model repository for later use. The repository is implemented by `Mozart.ProcessModelService`. When system users are ready to execute a process model, they retrieve it from the repository and use it to start a process engine execution.","ref":"app_overview.html#process-model-service"},{"type":"extras","title":"Processes Model Execution - Introduction to Mozart","doc":"Earlier we said that the job of the process engine is to drive a process model execution to completion. Let's discuss what that means.","ref":"app_overview.html#processes-model-execution"},{"type":"extras","title":"Completing Tasks - Introduction to Mozart","doc":"Every process model must specify an **initial task**. This will be the first task opened for completion. Completion of the initial task will often, though not always, trigger the opening of addition tasks. Every task that completes will potentially trigger the opening of additional tasks. At least one task in a process model will not trigger any downstream tasks. Eventually, a process model will stop generating new open tasks and it's process engine will terminate.\n\nSo, completing the execution of a process model equates to repeatedly opening and completing tasks until the process as a whole is complete.","ref":"app_overview.html#completing-tasks"},{"type":"extras","title":"Accumulating Data - Introduction to Mozart","doc":"Amost every task that is opened will need some data to complete. Further, when a task completes, it will introduce some new data for downstream tasks to use. Thus, completing a processes equates to generating a store of information.","ref":"app_overview.html#accumulating-data"},{"type":"extras","title":"Producing Side Effects - Introduction to Mozart","doc":"To be useful, almost all process model executions will produce some side effects. Take the process of processing a bank loan application for example. What are the possible side effects?\n\n* Sending a notice that the loan is approved or declined.\n* Depositing money in the customers account.\n* Setting up the loan in the banks backend system.","ref":"app_overview.html#producing-side-effects"},{"type":"extras","title":"Conclusion - Introduction to Mozart","doc":"Hopefully, the mechanics of Business Process Managment is now much clearer. In the newxt section, we will use Mozart to execute business processes.","ref":"app_overview.html#conclusion"},{"type":"extras","title":"First Process Execution","doc":"# First Process Execution\n\nNow we will put together what we've lerned so far to execute a process model.","ref":"first_process_execution.html"},{"type":"extras","title":"To Follow Along.. - First Process Execution","doc":"To follow along create a new Elxir project and then add Mozart as a dependency. Your dependency should look like this:\n\n```elixir\n  defp deps do\n    [\n      {:mozart, \"~> 0.1\"}\n    ]\n  end\n```","ref":"first_process_execution.html#to-follow-along"},{"type":"extras","title":"Construct the Process Model - First Process Execution","doc":"First, let us construct a simple process model that will have a single service task. The service task will simply add one to a data property. Here is the process model:\n\n```elixir\n    %ProcessModel{\n        name: :process_with_single_service_task,\n        tasks: [\n          %Service{\n            name: :service_task,\n            input_fields: [:x],\n            function: fn data -> Map.put(data, :x, data.x + 1) end\n          }\n        ],\n        initial_task: :service_task\n    }\n```\n\nThe process model definition is pretty simple. It uses two Mozart structures: `Mozart.Task.Service` and `Mozart.Data.ProcessModel`. The name of the process model is **:process_with_single_service_task**. It has just one service task which is named **:service_task**, and this task is the **initial_task** that will be opened upon upon process execution. The service task does not specify a next task so the process will complete after this task is completed.","ref":"first_process_execution.html#construct-the-process-model"},{"type":"extras","title":"Load the Process Model into repository - First Process Execution","doc":"Now let's load this process model into the process model repository (`Mozart.ProcessModelService`). First, let's paste a few aliases into our iex session:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessModelService, as: PMS\nalias Mozart.ProcessService, as: PS\nalias Mozart.Task.Service\nalias Mozart.Data.ProcessModel\n\n```\n\nNow we are ready to load our process model into the repository. Copy the following into your iex session:\n\n```elixir\nprocess_model = \n    %ProcessModel{\n        name: :process_with_single_service_task,\n        tasks: [\n          %Service{\n            name: :service_task,\n            input_fields: [:x],\n            function: fn data -> Map.put(data, :x, data.x + 1) end\n          }\n        ],\n        initial_task: :service_task\n    }\nPMS.load_process_model(process_model)\nPMS.get_process_model(:process_with_single_service_task)\n\n```\n\nFirst, we assigned a process model to a variable.\nSecondly, we loaded the process definition into our repository.\nFinally, we pulled the process model from the repository as verification. At the very bottom of the output, you should see the result of calling **get_process_model(:process_with_single_service_task)**:\n\n\n```\niex [12:52 :: 8] > PMS.get_process_model(:process_with_single_service_task)\n%Mozart.Data.ProcessModel{\n  name: :process_with_single_service_task,\n  tasks: [\n    %Mozart.Task.Service{\n      name: :service_task,\n      function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n      next: nil,\n      uid: nil,\n      data: %{},\n      type: :service\n    }\n  ],\n  initial_task: :service_task\n}\n```","ref":"first_process_execution.html#load-the-process-model-into-repository"},{"type":"extras","title":"Start a Process Engine and Execute the Process Model - First Process Execution","doc":"Now let us start a process engine, initializing it with our process model and some data. Then we call the function that will invoke process execution. Copy the following into your iex session:\n\n```elixir\n{:ok, pid, uid} = PE.start_process(:process_with_single_service_task, %{x: 0})\nPE.execute(pid)\n\n```\n\nYou should see something like:\n\n```\niex [12:22 :: 12] > {:ok, pid, uid} = PE.start_process(:process_with_single_service_task, %{x: 0})\n12:27:40.898 [info] Start process instance [process_with_single_service_task][0800de9a-8ec5-4906-bf50-bd09321f5982]\n{:ok, #PID<0.277.0>, \"0800de9a-8ec5-4906-bf50-bd09321f5982\"}\niex [12:22 :: 13] > PE.execute(pid)\n:ok\n12:27:40.899 [info] New task instance [service_task][da19b03a-009a-42aa-948c-4ad139d0fe66]\n12:27:40.899 [info] Complete service task [service_task[da19b03a-009a-42aa-948c-4ad139d0fe66]\n12:27:40.899 [info] Process complete [process_with_single_service_task][0800de9a-8ec5-4906-bf50-bd09321f5982]\n```\n\nWe created an instance of a process engine (`Mozart.ProcessEngine`) specifying the name of the process model to run and some initial data. Notice that the data includes the property **x** that our service task will use to do its computation.","ref":"first_process_execution.html#start-a-process-engine-and-execute-the-process-model"},{"type":"extras","title":"Verify the Results - First Process Execution","doc":"Let's verify the results of our process model execution. Copy the following into your iex session:\n\n```elixir\nPS.get_completed_process(uid)\n\n```\n\nand you should see:\n\n```elixir\niex [12:22 :: 14] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"0800de9a-8ec5-4906-bf50-bd09321f5982\",\n  parent: nil,\n  model_name: :process_with_single_service_task,\n  start_time: ~U[2024-05-28 17:27:40.898328Z],\n  end_time: ~U[2024-05-28 17:27:40.899505Z],\n  execute_duration: 1177,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      data: %{},\n      function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n      name: :service_task,\n      type: :service,\n      next: nil,\n      __struct__: Mozart.Task.Service,\n      uid: \"da19b03a-009a-42aa-948c-4ad139d0fe66\",\n      process_uid: \"0800de9a-8ec5-4906-bf50-bd09321f5982\"\n    }\n  ],\n  data: %{x: 1},\n  complete: true\n}\n```\n\nWe see the start and end times of process execution as well as the execution duration of 1177 microseconds (or 1.177 milliseconds). We see that tasks that were completed. And, finally, we see that the service task has done its job of adding 1 to the value of **x**.","ref":"first_process_execution.html#verify-the-results"},{"type":"extras","title":"Mozart Task Types","doc":"# Mozart Task Types","ref":"task_types.html"},{"type":"extras","title":"Properties Common to All Task Types - Mozart Task Types","doc":"All of the task types have the following properties:\n* **name**: an atome specifying the name of the task. Must be unique within the scope of the parent process model.\n* **next**: the name of the task that should be opened after completion of the current task. Exception: The task type `Mozart.Task.Paralled` doesn't have a next field. Instead, it has a multi_next field, which specifies next tasks that should be opened in parallel.\n* **uid**: When a task is opened (instantiated) it is assigned a unique identifier, i.e. its *uid*.\n* **type**: Specifies the type of the task. This value is defaulted automatically for each task type.","ref":"task_types.html#properties-common-to-all-task-types"},{"type":"extras","title":"Service Task - Mozart Task Types","doc":"A **Service Task** (`Mozart.Task.Service`) performs its work by calling an Elixir function. This function could perform a computation, call an external JSON service, retrieve data from a database, etc.\n\nA service task has two unique fields: **:function** and **:input_fields**.\n\n* The **:function** field specifies the function that the service task should apply for the purpose of returning output data into the process state.\n* The **:input_fields** field is used to select which process data fields are passed to the task's function. If no value is supplied for this field, the entire process data is passed.","ref":"task_types.html#service-task"},{"type":"extras","title":"Service Task example - Mozart Task Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Service\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessModelService, as: PMS\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow define a process model with a single service task and assign it to a variable:\n\n```\nmodel = %ProcessModel{\n    name: :process_with_single_service_task,\n    tasks: [\n      %Service{\n        name: :service_task,\n        input_fields: [:x, :y],\n        function: fn data -> Map.put(data, :sum, data.x + data.y) end\n      }\n    ],\n    initial_task: :service_task\n}\n\n```\n\nWe have specified that state data properties **:x** and **:y** should be available to the task's function. \n\nThe task function will take the sum of **x** and **y** and assign the result to a new the new property **sum**.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPMS.load_process_model(model)\ndata = %{x: 1, y: 1}\n{:ok, ppid, uid} = PE.start_process(:process_with_single_service_task, data)\nPE.execute(ppid)\n\n```\n\nNow, to view the state of the completed process, paste the following into your iex session:\n\n```\nPS.get_completed_process(uid)\n\n```\n\nand should see something like this:\n\n```\niex [12:21 :: 11] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"53d0220b-4e0c-42bb-9154-7e4aeff83837\",\n  parent: nil,\n  model_name: :process_with_single_service_task,\n  start_time: ~U[2024-05-30 17:22:49.156516Z],\n  end_time: ~U[2024-05-30 17:22:49.161447Z],\n  execute_duration: 4931,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n      name: :service_task,\n      type: :service,\n      next: nil,\n      __struct__: Mozart.Task.Service,\n      uid: \"443319db-9cef-47c0-9366-9b95ec1fa42b\",\n      input_fields: [:x, :y],\n      process_uid: \"53d0220b-4e0c-42bb-9154-7e4aeff83837\"\n    }\n  ],\n  data: %{sum: 2, y: 1, x: 1},\n  complete: true\n}\n```\n\nWe see that the new property **sum** has been added to the state of the completed process.","ref":"task_types.html#service-task-example"},{"type":"extras","title":"User Task - Mozart Task Types","doc":"A **User Task** (`Mozart.Task.User`) works by having a user complete some required task. This often takes the form of a user using a GUI to examine a subset of process data and then supplying additional data. \n\n**Important Note**: Users interact with a BPM platfrom such as Mozart by way of user application of some kind. The user application will allow the user to find tasks which may be assigned to him. Once the user accepts responsibility for a task, the application will then provide a user interface appropriate for accomplishing the given task. \n\nA user task has two unique fields: **:assigned_groups** and **:input_fields**.\n\n* The **:assigned_groups** field specifies the user groups that are elibigle to complete the task. \n* The **:input_fields** field is used to select which process data fields are passed to the user that will compleete the task. If no value is supplied for this field, the entire process data is passed.","ref":"task_types.html#user-task"},{"type":"extras","title":"User Task example - Mozart Task Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.User\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessModelService, as: PMS\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow we define a process model with a single service task and assign it to a variable:\n\n```\nmodel = %ProcessModel{\n      name: :user_task_process_model,\n      tasks: [\n        %User{\n          name: :user_task,\n          input_fields: [:x, :y],\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n  }\n\n```\n\nWe have specified that state data properties **:x** and **:y** should be available to the user that performs the task.\n\nThe user will compute the sum of **x** and **y** and assign the result to a new the new property **sum**.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPMS.load_process_model(model)\ndata = %{x: 1, y: 1}\n{:ok, ppid, uid} = PE.start_process(:user_task_process_model, data)\nPE.execute(ppid)\n\n```\n\nAt this point, a user task has been opened and is available for a user to claim and complete. Now we need to find a user task that can be claimed by persons in specified groups. We can do that like this:\n\n```\n[user_task] = PS.get_tasks_for_groups([\"admin\"])\n\n```\n\nwhich gives this result:\n\n```\n13:29:25.959 [info] New task instance [user_task][09fe2c87-d2c8-401b-a4df-83a7edab987d]\niex [13:28 :: 11] > PS.get_tasks_for_groups([\"admin\"])\n[\n  %{\n    complete: false,\n    data: %{y: 1, x: 1},\n    function: nil,\n    name: :user_task,\n    type: :user,\n    next: nil,\n    __struct__: Mozart.Task.User,\n    uid: \"09fe2c87-d2c8-401b-a4df-83a7edab987d\",\n    assigned_groups: [\"admin\"],\n    input_fields: [:x, :y],\n    process_uid: \"69b51421-cc10-45d7-8ca2-70c0e232cb82\"\n  }\n]\n```\n\nNow, pulling the task uid from the results above, we can complete the user task like this:\n\n```\nPE.complete_user_task(ppid,  user_task.uid, %{sum: 2})\n\n```\n\nwhich will produce a result like this:\n\n```\niex [16:54 :: 12] > PE.complete_user_task(ppid,  user_task.uid, %{sum: 2})\n16:56:11.417 [info] Complete user task [user_task][9b88be91-3b99-4e9c-b362-0f0dc5fb604c]\n16:56:11.417 [info] Process complete [user_task_process_model][d7bed372-94c4-477a-be0a-dcc7417c6e60]\n** (exit) exited in: GenServer.call(#PID<0.273.0>, {:complete_user_task, \"9b88be91-3b99-4e9c-b362-0f0dc5fb604c\", %{sum: 2}}, 5000)\n    ** (EXIT) shutdown\n    (elixir 1.16.2) lib/gen_server.ex:1114: GenServer.call/3\n    iex:12: (file)\n```\n\nDon't be concerned regarding the **shutdown** notice. It is normal. It simply means tht the process engine shutdown becuase the process had finished.\n\nNow we can verify that the process has completed and shows the expected value for process data:\n\n```\nPS.get_completed_process(uid)\n\n```\n\nand should see something like this:\n\n```\niex [13:28 :: 12] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"69b51421-cc10-45d7-8ca2-70c0e232cb82\",\n  parent: nil,\n  model_name: :user_task_process_model,\n  start_time: ~U[2024-05-30 18:29:25.954801Z],\n  end_time: ~U[2024-05-30 21:35:58.017106Z],\n  execute_duration: 11192062305,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      complete: false,\n      function: nil,\n      name: :user_task,\n      type: :user,\n      next: nil,\n      __struct__: Mozart.Task.User,\n      uid: \"09fe2c87-d2c8-401b-a4df-83a7edab987d\",\n      assigned_groups: [\"admin\"],\n      input_fields: [:x, :y],\n      process_uid: \"69b51421-cc10-45d7-8ca2-70c0e232cb82\"\n    }\n  ],\n  data: %{sum: 3, y: 1, x: 1},\n  complete: true\n}\n```\n\nWe see that the new property **sum** has been added to the state of the completed process.","ref":"task_types.html#user-task-example"},{"type":"extras","title":"Rule Task - Mozart Task Types","doc":"A **Rule Task** (`Mozart.Task.Rule`) performs its work by evaluating a rule table using process data. \n\nRule tasks use [Tablex](https://hexdocs.pm/tablex/0.3.1/readme.html), an incredibly useful Elixir library for evaluating [decision tables](https://en.wikipedia.org/wiki/Decision_table).\n\nA rule task has two unique fields: **:decision_args** and **:rule_table**.\n\n* The **:input_fields** field specifies the state data fields used to evaluate the rule table.\n* The **:rule_table** field holds the Tablex table definition.","ref":"task_types.html#rule-task"},{"type":"extras","title":"Rule Task example - Mozart Task Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Rule\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessModelService, as: PMS\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow define a process model with a single service task and assign it to a variable:\n\n```\nmodel = %ProcessModel{\n    name: :loan_approval,\n    tasks: [\n      %Rule{\n        name: :loan_decision,\n        input_fields: [:income],\n        rule_table:\n          Tablex.new(\"\"\"\n          F     income      || status\n          1     > 50000     || approved\n          2     <= 49999    || declined\n          \"\"\")\n      }\n    ],\n    initial_task: :loan_decision\n}\n\n```\n\nWe have specified that state data property **:income** should be available as an input field to the rule table.\n\nThe **:rule_table** property holds the Tablex table to be evaluated.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPMS.load_process_model(model)\ndata = %{income: 3000}\n{:ok, ppid, uid} = PE.start_process(:loan_approval, data)\nPE.execute(ppid)\n\n```\n\nNow, to view the state of the completed process, paste the following into your iex session:\n\n```\nPS.get_completed_process(uid)\n\n```\n\nand should see something like this:\n\n```\niex [20:45 :: 11] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"b79fe224-52b9-4457-b885-1234c47c0eb3\",\n  parent: nil,\n  model_name: :loan_approval,\n  start_time: ~U[2024-06-01 01:45:57.687401Z],\n  end_time: ~U[2024-06-01 01:45:57.695101Z],\n  execute_duration: 7700,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      name: :loan_decision,\n      type: :rule,\n      next: nil,\n      __struct__: Mozart.Task.Rule,\n      uid: \"a045ccfe-21d1-410f-aa87-6e020017e48d\",\n      input_fields: [:income],\n      rule_table: %Tablex.Table{\n        hit_policy: :first_hit,\n        inputs: [\n          %Tablex.Variable{\n            name: :income,\n            label: \"income\",\n            desc: nil,\n            type: :undefined,\n            path: []\n          }\n        ],\n        outputs: [\n          %Tablex.Variable{\n            name: :status,\n            label: \"status\",\n            desc: nil,\n            type: :undefined,\n            path: []\n          }\n        ],\n        rules: [\n          [1, {:input, [>: 50000]}, {:output, [\"approved\"]}],\n          [2, {:input, [<=: 49999]}, {:output, [\"declined\"]}]\n        ],\n        valid?: :undefined,\n        table_dir: :h\n      },\n      process_uid: \"b79fe224-52b9-4457-b885-1234c47c0eb3\"\n    }\n  ],\n  data: %{status: \"declined\", income: 3000},\n  complete: true\n}\n```\n\nWe see that the new property **status** with a value of **declined** has been added to the state of the completed process.","ref":"task_types.html#rule-task-example"},{"type":"extras","title":"Parallel Task - Mozart Task Types","doc":"","ref":"task_types.html#parallel-task"},{"type":"extras","title":"Subprocess Task - Mozart Task Types","doc":"","ref":"task_types.html#subprocess-task"},{"type":"extras","title":"Timer Task - Mozart Task Types","doc":"","ref":"task_types.html#timer-task"},{"type":"extras","title":"Join Task - Mozart Task Types","doc":"","ref":"task_types.html#join-task"},{"type":"extras","title":"Send Task - Mozart Task Types","doc":"","ref":"task_types.html#send-task"},{"type":"extras","title":"Receive Task - Mozart Task Types","doc":"","ref":"task_types.html#receive-task"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}